import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import java.lang.Thread;

class Poofz implements Runnable
{
	public static final int PIXEL_WIDTH = 100;
	public static final int PIXEL_HEIGHT = 100;
	public static final int CANVAS_ZOOM = 4;
	private PoofzCanvas canvas;
	private boolean keep_running;

	public Poofz()
	{
		JFrame frame = new JFrame("POOFZ");
		canvas = new PoofzCanvas();
		canvas.setSize(PIXEL_WIDTH*CANVAS_ZOOM,PIXEL_HEIGHT*CANVAS_ZOOM);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().add(canvas, BorderLayout.CENTER);
		frame.getContentPane().add(new PoofzButtons(this), BorderLayout.SOUTH);
		frame.pack();
		frame.setVisible(true);
	}

	public static void main(String[] args)
	{
		Poofz poofz = new Poofz();
	}

	public void stop()
	{
		keep_running = false;
	}

	public void run()
	{
		keep_running = true;
		int x1, x2, y1, y2, x, y;
		while(keep_running)
		{
			x1 = (int)Math.floor(Math.random()*PIXEL_WIDTH*2);
			x2 = (int)Math.floor(Math.random()*PIXEL_WIDTH*2);
			y1 = (int)Math.floor(Math.random()*PIXEL_HEIGHT*2);
			y2 = (int)Math.floor(Math.random()*PIXEL_HEIGHT*2);
			x1-= PIXEL_WIDTH/2;
			x2-= PIXEL_WIDTH/2;
			y1-= PIXEL_HEIGHT/2;
			y2-= PIXEL_HEIGHT/2;
			if(x1<0) x1=0;
			if(x2<0) x2=0;
			if(y1<0) y1=0;
			if(y2<0) y2=0;
			if(x1>=PIXEL_WIDTH) x1=PIXEL_WIDTH-1;
			if(x2>=PIXEL_WIDTH) x2=PIXEL_WIDTH-1;
			if(y1>=PIXEL_HEIGHT) y1=PIXEL_HEIGHT-1;
			if(y2>=PIXEL_HEIGHT) y2=PIXEL_HEIGHT-1;
			x = (int)Math.floor(Math.random()*(x2-x1))+x1;
			y = (int)Math.floor(Math.random()*(y2-y1))+y1;
			canvas.poofz(x,y,x1,y1,x2,y2);
			canvas.paint();
			Thread.currentThread().yield();
		}
	}
}

class PoofzCanvas extends Canvas
{
	int[][] reds = new int[Poofz.PIXEL_WIDTH][Poofz.PIXEL_HEIGHT];
	int[][] grns = new int[Poofz.PIXEL_WIDTH][Poofz.PIXEL_HEIGHT];
	int[][] blus = new int[Poofz.PIXEL_WIDTH][Poofz.PIXEL_HEIGHT];

	public PoofzCanvas()
	{
		for(int r=0; r<Poofz.PIXEL_WIDTH; r++)
		{
			for(int c=0; c<Poofz.PIXEL_HEIGHT; c++)
			{
				reds[r][c] = (int)Math.floor(Math.random()*256);
				grns[r][c] = 0; //(int)Math.floor(Math.random()*256);
				blus[r][c] = 0; //(int)Math.floor(Math.random()*256);
			}
		}
	}

	public void paint()
	{
		paint(this.getGraphics());
	}

	public void paint(Graphics g)
	{
		Color color;

		for(int r=0; r<Poofz.PIXEL_WIDTH; r++)
		{
			for(int c=0; c<Poofz.PIXEL_HEIGHT; c++)
			{
				color = new Color(reds[r][c], grns[r][c], blus[r][c]);
				g.setColor(color);
				g.fillRect(r*Poofz.CANVAS_ZOOM,c*Poofz.CANVAS_ZOOM,Poofz.CANVAS_ZOOM,Poofz.CANVAS_ZOOM);
			}
		}
	}

	private void poofz_red(int x1, int y1, int x2, int y2)
	{
		int[] freqs = new int[256];
		int hi_count = 0;
		for(int r=x1; r<=x2; r++)
		{
			for(int c=y1; c<=y2; c++)
			{
				freqs[reds[r][c]]++;
				if(freqs[reds[r][c]] > hi_count) hi_count = freqs[reds[r][c]];
			}
		}
		int[] colors = new int[256];
		for(int i=1; i<256; i++) colors[i] = i;
		boolean done = false;
		int t;
		while(!done)
		{
			done = true;
			for(int i=1; i<256; i++)
			{
				if(freqs[i] > freqs[i-1])
				{
					t = freqs[i];
					freqs[i] = freqs[i-1];
					freqs[i-1] = t;
					t = colors[i];
					colors[i] = colors[i-1];
					colors[i-1] = t;
					done = false;
				}
			}
		}
		int hi_color = 0;
		double hi_diff = -99999;
		int lo_color = 0;
		double lo_diff = 99999;
		double expected = 0;
		double diff = 0;
		for(int i=1; i<256; i++)
		{
			expected = freqs[0]/(i+1);
			diff = freqs[i] - expected;
			if(diff > hi_diff)
			{
				hi_diff = diff;
				hi_color = colors[i];
			}
			if(diff < lo_diff)
			{
				lo_diff = diff;
				lo_color = colors[i];
			}
		}

		int hi_r = 0;
		int lo_r = 0;
		int hi_c = 0;
		int lo_c = 0;
		for(int r=x1; r<=x2; r++)
		{
			for(int c=y1; c<=y2; c++)
			{
				if(reds[r][c] == hi_color)
				{
					hi_r = r;
					hi_c = c;
				}
				if(reds[r][c] == lo_color)
				{
					lo_r = r;
					lo_c = c;
				}
			}
		}
		reds[hi_r][hi_c] = reds[lo_r][lo_c];
	}

	public void poofz(int x, int y, int x1, int y1, int x2, int y2)
	{
		if(Math.random()>2)
		{
			reds[x][y] = (int)Math.floor(Math.random()*256);
//			grns[x][y] = (int)Math.floor(Math.random()*256);
//			blus[x][y] = (int)Math.floor(Math.random()*256);
		}
		else
		{
			int t;
			if(x1 > x2)
			{
				t=x1;
				x1 = x2;
				x2 = t;
			}
			if(y1 > y2)
			{
				t=y1;
				y1 = y2;
				y2 = t;
			}
			poofz_red(x1, y1, x2, y2);
		}
	}
}

class PoofzButtons extends JPanel implements ActionListener
{
	private JButton start;
	private JButton stop;
	private Poofz poofz;

	public PoofzButtons(Poofz p)
	{
		poofz = p;
		start = new JButton("Start");
		stop = new JButton("Stop");
		this.setLayout(new GridLayout(1,2));
		this.add(start);
		this.add(stop);
		start.addActionListener(this);
		stop.addActionListener(this);
	}

	public void actionPerformed(ActionEvent e)
	{
		if(e.getActionCommand() == "Start")
		{
			new Thread(poofz).start();
		}
		else poofz.stop();
	}
}

